use std::fmt;

use crate::Token;

pub enum LispValue {
	Nil,
	String(String),
	LispFunction(String, Vec<OpCode>),
	RustFunction(String, fn(Vec<LispValue>) -> LispValue)
}

impl fmt::Display for LispValue {
	fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
		match self {
			LispValue::Nil => write!(f, "nil"),
			LispValue::String(str) => write!(f, "{}", str),
			LispValue::LispFunction(name, _) => write!(f, "<'{}': Lisp Function>", name),
			LispValue::RustFunction(name, _) => write!(f, "<'{}': Rust Function>", name)
		}
	}
}

impl From<&str> for LispValue {
	fn from(value: &str) -> Self {
		LispValue::String(value.to_string())
	}
}

impl From<String> for LispValue {
	fn from(value: String) -> Self {
		LispValue::String(value)
	}
}

pub enum OpCode {
	Call(String, Vec<LispValue>),
	Eval(Vec<OpCode>)
}



pub fn parse(tokens: Vec<Token>) -> Vec<OpCode> {
	let mut opcodes = Vec::new();

	// TODO:
	opcodes.push(OpCode::Call("print".to_string(), vec![
		LispValue::from("Hello, World!")
	]));

	opcodes
}